require "jma/certtool/ca"
require "jma/certtool/openssl"

module JMA::CertTool::CA::Profile
  CA_PROFILE = OpenSSL::Config.parse(<<-_end_of_profile_)
    [default]
    name             = sub-ca
    summary          = Subordinate Certificate Authority
    key_algorithm    = RSA
    key_length       = 2048
    digest_algorithm = SHA256
    validity         = 3650

    [extensions]
    basicConstraints       = critical, CA:TRUE, pathlen:0
    keyUsage               = critical, keyCertSign, cRLSign
    subjectKeyIdentifier   = hash
    authorityKeyIdentifier = keyid:always
    subjectAltName         = copy_from_request
  _end_of_profile_

  TEMPLATE = OpenSSL::Config.parse(<<-_end_of_profile_)
    [default]
    name             = <name>
    summary          = <summary>
    key_algorithm    = RSA
    key_length       = 2048
    digest_algorithm = SHA256
    validity         = 365

    [extensions]
    keyUsage               = <keyUsage>
    extendedKeyUsage       = <extKeyUsage>
    subjectKeyIdentifier   = hash
    authorityKeyIdentifier = keyid:always
    subjectAltName         = copy_from_request
  _end_of_profile_

  SSL_CLIENT_PROFILE = TEMPLATE.dup
  SSL_CLIENT_PROFILE.name = "ssl-client"
  SSL_CLIENT_PROFILE.summary = "SSL Client Certificate"
  SSL_CLIENT_PROFILE.add_value("extensions", "extendedKeyUsage", "clientAuth")
  SSL_CLIENT_PROFILE.add_value(
    "extensions", "keyUsage", "critical, digitalSignature")

  SSL_SERVER_PROFILE = TEMPLATE.dup
  SSL_SERVER_PROFILE.name = "ssl-server"
  SSL_SERVER_PROFILE.summary = "SSL Server Certificate"
  SSL_SERVER_PROFILE.add_value("extensions", "extendedKeyUsage", "serverAuth")
  SSL_SERVER_PROFILE.add_value(
    "extensions", "keyUsage", "critical, digitalSignature, keyEncipherment")

  USER_PROFILE = TEMPLATE.dup
  USER_PROFILE.name = "user"
  USER_PROFILE.summary = "Internet User Certificate (SSL & S/MIME)"
  USER_PROFILE.add_value(
    "extensions", "keyUsage", "critical, digitalSignature, keyEncipherment")
  USER_PROFILE.add_value(
    "extensions", "extendedKeyUsage", "clientAuth, emailProtection")

  CODE_SIGN_PROFILE = TEMPLATE.dup
  CODE_SIGN_PROFILE.name = "code-signing"
  CODE_SIGN_PROFILE.summary = "Code Signing Certificate"
  CODE_SIGN_PROFILE.add_value(
    "extensions", "extendedKeyUsage", "codeSigning")
  CODE_SIGN_PROFILE.add_value(
    "extensions", "keyUsage", "critical, digitalSignature")
end
